#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from typing import Optional
from pprint import pprint

from .config import FD, Tree, Key
from .utils import FileIO

from .run import PageDataTree


def main(input_filepath: str, output_filepath: str, key: str, tree: str) -> Optional[NotImplementedError]:
    file_io = FileIO(input_filepath)
    file_data = file_io.load()

    pdt = PageDataTree(file_data)

    if key:
      pdt_tree = pdt.tree_by_key(
        key=key,
        result_to=(Key.SAVE if output_filepath else Key.SHOW)
      )
    else:
      pdt_tree = (tree,)

    for tree in pdt_tree:
      if not tree:
        continue

      tree_data = pdt.data_by_tree(tree)

      if output_filepath:
        if output_filepath == FD.STDOUT:
          print(tree)
          pprint(tree_data)
          print()
          continue

        if key:
          raise NotImplementedError('Use `-t` flag to dump a value for `tree`')

        file_io.dump(tree_data, output_filepath)


if __name__ == '__main__':
  parser = argparse.ArgumentParser(description='JSON Tree parser')
  parser.add_argument('-i', type=str, help='[i]nput filepath')
  parser.add_argument('-o', type=str, help='[o]utput filepath')
  parser.add_argument('-k', type=str, help='[k]ey to search (generate) tree for')
  parser.add_argument('-t', type=str, help='[t]ree to save the data from')
  parser.add_argument('-l', type=int, help='[l]limit stdout')
  parser.add_argument('-fk', type=str, help='[f]ilter stdout by must have `key`')
  parser.add_argument('-fv', type=str, help='[f]ilter stdout by must have `value`')
  args = parser.parse_args()

  input_filepath = args.i
  output_filepath = args.o
  key = args.k
  tree = args.t

  if not input_filepath:
    print('Input filepath required')
    exit()
  if (not key) and (not tree):
    print('Key or Tree is required')
    exit()

  if args.l:
    # decrements the limit befor returning the `tree`
    Tree.SEARCH_LIMIT = (args.l+1)

  Tree.SEARCH_FILTER_KEY = args.fk or Tree.SEARCH_FILTER_KEY
  Tree.SEARCH_FILTER_VALUE = args.fv or Tree.SEARCH_FILTER_VALUE

  main(input_filepath, output_filepath, key, tree)



# TODO: use namedtuple


class FD:
  """File and Directory
  """
  ROOT_DIRECTORY = 'data'
  FILE_EXTENSION = '.json'
  STDOUT = '*'


class Tree:
  ROOT = 'root'
  DELIMITER = ' -> '
  LIST_INDEX_WRAPPER = "[{index}]"
  SEARCH_LIMIT = -1
  SEARCH_FILTER_KEY = ''
  SEARCH_FILTER_VALUE = ''


class Key:
  SHOW = 'print'
  SAVE = 'return'



import os
import json
from typing import Optional, Union, NoReturn, Any, Generator

from .config import Tree, Key
from .utils import FileIO


# TODO
#   - Rename *PageDataTree


class PageDataTree:

  def __init__(self, data: json):
    self.data = data

  # TODO: check for other types
  @staticmethod
  def join_tree(tree: str, key: str, list_index: Optional[int] = None):
    keys = [tree]

    if list_index is not None:
      keys.append(Tree.LIST_INDEX_WRAPPER.format(index=list_index))

    keys.append(key)

    return Tree.DELIMITER.join(keys)

  def process_result(self, result: str,
                           result_to: str,
                           result_value: str) -> Union[Optional[str], NoReturn]:
    # TODO: rename parameters name
    if Tree.SEARCH_FILTER_KEY and (Tree.SEARCH_FILTER_KEY not in result):
      return None

    if Tree.SEARCH_FILTER_VALUE:
      if not isinstance(result_value, (int, str)):
        return None
      if str(result_value) != Tree.SEARCH_FILTER_VALUE:
        return None

    if (Tree.SEARCH_LIMIT and Tree.SEARCH_LIMIT > 0):
      Tree.SEARCH_LIMIT -= 1

    if Tree.SEARCH_LIMIT == 0:
      exit()

    if result_to == Key.SHOW:
      print(result)

    if result_to == Key.SAVE:
      return result

  def tree_by_key(self, data: Optional[dict] = None, key: str = '',
                        gen_tree: str = Tree.ROOT, list_index: Optional[int] = None,
                        result_to: str = Key.SAVE) -> Generator[str, None, None]:
    """Yields str: ex. A -> B -> C -> *key
    """
    if data is None:
      data = self.data

    if isinstance(data, list):
      for idx, item in enumerate(data):
        yield from self.tree_by_key(
          item, key, gen_tree, list_index=idx, result_to=result_to
        )

    if isinstance(data, dict):
      for data_key, data_value in data.items():
        tree = self.join_tree(gen_tree, data_key, list_index)

        if data_key == key:
          yield self.process_result(tree, result_to, data_value)
          continue

        if data_key is None:
          continue
        if data_value is None:
          continue

        yield from self.tree_by_key(
          data_value, key, tree, result_to=result_to
        )

  def data_by_tree(self, tree: str) -> dict:
    if Tree.ROOT:
      tree = tree.replace(
        self.join_tree(Tree.ROOT, ''), '', 1
      )

    tree_keys = tree.split(Tree.DELIMITER)
    inner_data = self.data

    for key in tree_keys:
      if key.startswith(Tree.LIST_INDEX_WRAPPER[0]):
        inner_data = inner_data[int(key[1:-1])]
      else:
        inner_data = inner_data[key]

    return inner_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup
from jsontp.__init__ import __version__

setup(
  name='jsontp',
  version=__version__,
  description='JSON Tree Parser',
  long_description='Script to generate a json `tree` for `key` and dumping the `value` of the `key` by `tree`',
  author='Nodiru Gaji',
  author_email='c0d3r.nodiru.gaji@gmail.com',
  url='https://github.com/ames0k0/jsontree',
  packages=['jsontp'],
  license='GNU General Public License v3 (GPLv3)',
  classifiers=[
    'Programming Language :: Python :: 3.10',
  ],
  platforms=[
    'Operating System :: OS Independent',
  ],
)
